package org.processmining.starstar.elements;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.starstar.model.ModelEdge;

public class XOCActivity {
	String activityLabel;
	List<XOCEvent> subscribedEvents;
	List<ModelEdge> inputModelEdges;
	List<ModelEdge> outputModelEdges;
	Integer activityCount;
	public Set<XOCObject> objectsInSubscribedEvents;
	public Map<String, Set<XOCObject>> inputRelationshipObjects;
	public Map<String, Set<XOCObject>> outputRelationshipObjects;
	public Map<String, Map<String, Set<XOCObject>>> inputRelationshipObjectsClassMap;
	public Map<String, Map<String, Set<XOCObject>>> outputRelationshipObjectsClassMap;
	
	public XOCActivity(String activityLabel) {
		this.activityLabel = activityLabel;
		this.subscribedEvents = new ArrayList<XOCEvent>();
		this.inputModelEdges = new ArrayList<ModelEdge>();
		this.outputModelEdges = new ArrayList<ModelEdge>();
		this.activityCount = 0;
		this.objectsInSubscribedEvents = new HashSet<XOCObject>();
		this.inputRelationshipObjects = new HashMap<String, Set<XOCObject>>();
		this.outputRelationshipObjects = new HashMap<String, Set<XOCObject>>();
		this.inputRelationshipObjectsClassMap = new HashMap<String, Map<String, Set<XOCObject>>>();
		this.outputRelationshipObjectsClassMap = new HashMap<String, Map<String, Set<XOCObject>>>();
	}
	
	public Integer getActivityCount() {
		return this.activityCount;
	}
	
	public void setActivityCount(Integer activityCount) {
		this.activityCount = activityCount;
	}
	
	public void increaseActivityCount() {
		this.activityCount++;
	}
	
	public String getActivityLabel() {
		return this.activityLabel;
	}
	
	public List<XOCEvent> getSubscribedEvents() {
		return this.subscribedEvents;
	}
	
	public void addSubscribedEvent(XOCEvent event) {
		this.addSubscribedEvent(event, true);
	}
	
	public void addSubscribedEvent(XOCEvent event, boolean increaseCount) {
		this.subscribedEvents.add(event);
		if (increaseCount) {
			this.increaseActivityCount();
		}
	}
	
	public List<ModelEdge> getInputModelEdges() {
		return this.inputModelEdges;
	}
	
	public void addInputModelEdge(ModelEdge edge) {
		this.inputModelEdges.add(edge);
	}
	
	public List<ModelEdge> getOutputModelEdges() {
		return this.outputModelEdges;
	}
	
	public void addOutputModelEdge(ModelEdge edge) {
		this.outputModelEdges.add(edge);
	}
	
	public void calculateObjectsInSubscribedEvents() {
		for (XOCEvent event : this.subscribedEvents) {
			this.objectsInSubscribedEvents.addAll(event.getReferredObjects());
		}
	}
	
	public void calculateRelationshipObjects() {
		for (ModelEdge edge : this.inputModelEdges) {
			String activityName = edge.getSourceActivity().getActivityLabel();
			Set<XOCObject> associatedObjects = new HashSet<XOCObject>(edge.getAssociatedObjects());
			this.inputRelationshipObjects.put(activityName, associatedObjects);
			this.inputRelationshipObjectsClassMap.put(activityName, new HashMap<String, Set<XOCObject>>());
			for (XOCObject object : associatedObjects) {
				String objectClass = object.getObjectClass().getClassLabel();
				if (!this.inputRelationshipObjectsClassMap.get(activityName).containsKey(objectClass)) {
					this.inputRelationshipObjectsClassMap.get(activityName).put(objectClass, new HashSet<XOCObject>());
				}
				this.inputRelationshipObjectsClassMap.get(activityName).get(objectClass).add(object);
			}
		}
		for (ModelEdge edge : this.outputModelEdges) {
			String activityName = edge.getTargetActivity().getActivityLabel();
			Set<XOCObject> associatedObjects = new HashSet<XOCObject>(edge.getAssociatedObjects());
			this.outputRelationshipObjects.put(activityName, associatedObjects);
			this.outputRelationshipObjectsClassMap.put(activityName, new HashMap<String, Set<XOCObject>>());
			for (XOCObject object : associatedObjects) {
				String objectClass = object.getObjectClass().getClassLabel();
				if (!this.outputRelationshipObjectsClassMap.get(activityName).containsKey(objectClass)) {
					this.outputRelationshipObjectsClassMap.get(activityName).put(objectClass, new HashSet<XOCObject>());
				}
				this.outputRelationshipObjectsClassMap.get(activityName).get(objectClass).add(object);
			}
		}
	}
	
	@Override
	public String toString() {
		return this.activityLabel;
	}
	
	/*public void printRelationshipObjects() {
		for (String activityName : this.outputRelationshipObjectsClassMap.keySet()) {
			for (String className : this.outputRelationshipObjectsClassMap.get(activityName).keySet()) {
				for (XOCObject relatedObject : this.outputRelationshipObjectsClassMap.get(activityName).get(className)) {
					System.out.println(this.activityLabel + " " + activityName+" "+className+" "+relatedObject.getObjectName());
				}
			}
		}
	}*/
}
