package org.processmining.starstar.elements;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.starstar.model.EventEdge;

public class XOCEvent {
	String eventId;
	Date eventTimestamp;
	XOCActivity activity;
	String activityName;
	List<XOCObject> referredObjects;
	Map<String, XOCObject> referredObjectsMap;
	List<XOCObjectVersion> xocObjectVersions;
	List<XOCRelation> relations;
	List<EventEdge> comingEdges;
	List<EventEdge> outgoingEdges;
	
	public XOCEvent(String eventId, Date eventTimestamp, XOCActivity xocActivity) {
		this.eventId = eventId;
		this.eventTimestamp = eventTimestamp;
		this.referredObjects = new ArrayList<XOCObject>();
		this.referredObjectsMap = new HashMap<String, XOCObject>();
		this.xocObjectVersions = new ArrayList<XOCObjectVersion>();
		this.relations = new ArrayList<XOCRelation>();
		this.activity = xocActivity;
		this.activityName = xocActivity.getActivityLabel();
		this.activity.addSubscribedEvent(this);
		this.comingEdges = new ArrayList<EventEdge>();
		this.outgoingEdges = new ArrayList<EventEdge>();
	}
	
	public XOCActivity getActivity() {
		return this.activity;
	}
	
	public String getActivityName() {
		return this.activityName;
	}
	
	public List<XOCRelation> getRelations() {
		return this.relations;
	}
	
	public void addRelation(XOCRelation relation) {
		this.relations.add(relation);
	}
	
	public String getEventId() {
		return this.eventId;
	}
	
	public Date getTimestamp() {
		return this.eventTimestamp;
	}
	
	public void addReferredObject(XOCObject referredObject) {
		if (!this.referredObjects.contains(referredObject)) {
			referredObject.addSubscribedEvent(this);
			this.referredObjects.add(referredObject);
			this.referredObjectsMap.put(referredObject.getObjectName(), referredObject);
		}
	}
	
	public List<XOCObject> getReferredObjects() {
		return this.referredObjects;
	}
	
	public void addXocObjectVersion(XOCObjectVersion xocObjectVersion) {
		this.xocObjectVersions.add(xocObjectVersion);
		if (this.referredObjectsMap.containsKey(xocObjectVersion.getLabel())) {
			this.referredObjectsMap.get(xocObjectVersion.getLabel()).addXocObjectVersion(xocObjectVersion);
			xocObjectVersion.setCorrespondingObject(this.referredObjectsMap.get(xocObjectVersion.getLabel()));
		}
	}
	
	public List<XOCObjectVersion> getXOCObjectVersion() {
		return this.xocObjectVersions;
	}
	
	public String getShortResume() {
		StringBuilder ret = new StringBuilder();
		ret.append("eventId = "+eventId);
		ret.append(" eventTimestamp = "+eventTimestamp);
		ret.append(" activity = "+activity.getActivityLabel());
		ret.append("\n");
		return ret.toString();
	}
	
	public String getResume() {
		StringBuilder ret = new StringBuilder();
		ret.append("eventId = "+eventId);
		ret.append("\neventTimestamp = "+eventTimestamp);
		ret.append("\nactivity = "+activity.getActivityLabel()+"\n");
		for (XOCObject object : referredObjects) {
			ret.append(object.getShortResume());
		}
		for (XOCRelation relation : relations) {
			ret.append(relation.getResume());
		}
		return ret.toString();
	}
	
	public void printResume() {
		System.out.println(getResume());
	}
	
    @Override
    public String toString() {
        return getShortResume();
    }
	
	public List<EventEdge> getComingEdges() {
		return this.comingEdges;
	}
	
	public void addComingEdge(EventEdge edge) {
		this.comingEdges.add(edge);
	}
	
	public List<EventEdge> getOutgoingEdges() {
		return this.outgoingEdges;
	}
	
	public void addOutgoingEdge(EventEdge edge) {
		this.outgoingEdges.add(edge);
	}
}