package org.processmining.starstar.elements;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.processmining.starstar.constants.ModelConstants;

public class XOCObject {
	String objectName;
	XOCObjectClass objectClass;
	String objectClassName;
	boolean isTimed;
	List<XOCObjectVersion> xocObjectVersions;
	List<XOCEvent> subscribedEvents;

	// memorize the address to this map
	Map<String, XOCObjectClass> xocObjectClasses;
	
	public XOCObject(String objectName, Map<String, XOCObjectClass> xocObjectClasses) {
		this.objectName = objectName;
		this.xocObjectVersions = new ArrayList<XOCObjectVersion>();
		this.subscribedEvents = new ArrayList<XOCEvent>();
		this.xocObjectClasses = xocObjectClasses;
		this.isTimed = false;
	}
	
	public String getObjectName() {
		return this.objectName;
	}
	
	public XOCObjectClass getObjectClass() {
		return this.objectClass;
	}
	
	public void setObjectClass(XOCObjectClass objectClass) {
		if (!xocObjectClasses.containsKey(objectClass.getClassLabel())) {
			xocObjectClasses.put(objectClass.getClassLabel(), objectClass);
		}
		this.objectClass = objectClass;
		this.objectClassName = objectClassName;
		this.objectClass.subscribeObject(this);
	}
	
	public List<XOCObjectVersion> getXocObjectVersions() {
		return this.xocObjectVersions;
	}
	
	public void addXocObjectVersion(XOCObjectVersion version) {
		this.xocObjectVersions.add(version);
		if (version.getObjectClass() != null) {
			this.setObjectClass(version.getObjectClass());
		}
		if (version.isTimed) {
			this.isTimed = true;
		}
	}
	
	public List<XOCEvent> getSubscribedEvents() {
		return this.subscribedEvents;
	}
	
	public void addSubscribedEvent(XOCEvent event) {
		if (!this.subscribedEvents.contains(event)) {
			this.subscribedEvents.add(event);
		}
	}
	
	public double getWeight() {
		if (this.subscribedEvents.size() > 0) {
			if (isTimed) {
				return 1.0 / (new Double(this.subscribedEvents.size()));
			}
			return ModelConstants.decorationImportance / (new Double(this.subscribedEvents.size()));
		}
		if (isTimed) {
			return 1.0;
		}
		return ModelConstants.decorationImportance;
	}
	
	public String getShortResume() {
		StringBuilder ret = new StringBuilder();
		ret.append("objectName: "+this.objectName+" ");
		if (this.objectClass != null) {
			ret.append("objectClass: "+this.objectClass.getClassLabel());
		}
		ret.append("\n");
		return ret.toString();
	}
	
	public String getResume() {
		StringBuilder ret = new StringBuilder();
		ret.append("objectName: "+this.objectName+"\n");
		if (this.objectClass != null) {
			ret.append("objectClass: "+this.objectClass.getClassLabel()+"\n");
		}
		ret.append("xocObjectVersions=\n");
		for (XOCObjectVersion objectVersion : this.xocObjectVersions) {
			ret.append(objectVersion.getResume());
		}
		ret.append("subscribedEvents=\n");
		for (XOCEvent event : this.subscribedEvents) {
			ret.append(event.getShortResume());
		}
		ret.append("weight="+this.getWeight()+"\n");
		
		return ret.toString();
	}
	
	public void printResume() {
		System.out.println(getResume());
	}
	
    @Override
    public String toString() {
        return getShortResume();
    }
}
