package org.processmining.starstar.logExtraction;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.processmining.starstar.elements.XOCEvent;
import org.processmining.starstar.elements.XOCObject;
import org.processmining.starstar.utilities.XOCEventComparator;
import org.processmining.starstar.utilities.XOCObjectResumeComparator;

public class StarStarTrace {
	Map<XOCObject, List<XOCObject>> intersectedObjects;
	List<XOCObject> involvedObjects;
	List<XOCEvent> involvedEvents;
	
	public StarStarTrace(Map<XOCObject, List<XOCObject>> intersectedObjects, XOCObject startingObject) {
		this.intersectedObjects = intersectedObjects;
		this.involvedObjects = new ArrayList<XOCObject>();
		this.involvedEvents = new ArrayList<XOCEvent>();
		this.addInvolvedObject(startingObject);
	}
	
	public void addInvolvedObject(XOCObject object) {
		this.involvedObjects.add(object);
	}
	
	public void sortObjectsByResume() {
		Collections.sort(this.involvedObjects, new XOCObjectResumeComparator());
	}
	
	public void sortEventsByTimestamp() {
		Collections.sort(this.involvedEvents, new XOCEventComparator());
	}
	
	public List<XOCObject> getInvolvedObjects() {
		return this.involvedObjects;
	}
	
	public String getStarStarObjectsRepresentation() {
		this.sortObjectsByResume();
		StringBuilder ret = new StringBuilder();
		for (XOCObject involvedObject : involvedObjects) {
			ret.append(involvedObject.getShortResume());
		}
		return ret.toString();
	}
	
	public void formInvolvedEvents() {
		this.sortObjectsByResume();
		this.involvedEvents = new ArrayList<XOCEvent>();
		for (XOCObject involvedObject : this.involvedObjects) {
			List<XOCEvent> events = involvedObject.getSubscribedEvents();
			for (XOCEvent eve : events) {
				if (!this.involvedEvents.contains(eve)) {
					this.involvedEvents.add(eve);
				}
			}
		}
	}
	
	public List<XOCEvent> getInvolvedEvents() {
		return this.involvedEvents;
	}
	
	public String getStarStarEventsRepresentation() {
		this.sortEventsByTimestamp();
		StringBuilder ret = new StringBuilder();
		for (XOCObject involvedObject : involvedObjects) {
			ret.append(involvedObject.getShortResume());
		}
		return ret.toString();
	}
	
	public void printTraceRepresentation(String reason) {
		formInvolvedEvents();
		this.sortEventsByTimestamp();
		System.out.println("string representation (reason: "+reason+"):");
		System.out.println("objects: ");
		for (XOCObject involvedObject : involvedObjects) {
			System.out.print(involvedObject.getShortResume());
		}
		System.out.println("events: ");
		for (XOCEvent event : involvedEvents) {
			System.out.println(event.getShortResume());
		}
		System.out.println("");
	}
}
